@@ -5,7 +5,7 @@ use warnings;
 use Carp;
 use Cwd 'realpath';
 
-our $VERSION   = '1.3089_01';
+our $VERSION   = '1.3095_01';
 our $AUTHORITY = 'SUKRIA';
 
 use Dancer::App;
@@ -49,9 +49,11 @@ our @EXPORT    = qw(
   config
   content_type
   dance
+  dancer_version
   debug
   del
   dirname
+  info
   error
   engine
   false
@@ -134,6 +136,7 @@ sub cookies         { Dancer::Cookies->cookies }
 sub config          { Dancer::Config::settings() }
 sub content_type    { Dancer::SharedData->response->content_type(@_) }
 sub dance           { goto &start }
+sub dancer_version  { Dancer->VERSION }
 sub debug           { goto &Dancer::Logger::debug }
 sub del             { Dancer::App->current->registry->universal_add('delete',  @_) }
 sub dirname         { Dancer::FileUtils::dirname(@_) }
@@ -154,6 +157,7 @@ sub halt            { Dancer::SharedData->response->halt(@_);
                       Dancer::Continuation::Halted->new->throw;
                     }
 sub header          { goto &headers }
+sub info            { goto &Dancer::Logger::info }
 sub push_header     { Dancer::SharedData->response->push_header(@_); }
 sub headers         { Dancer::SharedData->response->headers(@_); }
 sub hook            { Dancer::Hook->new(@_) }
@@ -179,7 +183,8 @@ sub pass            { Dancer::SharedData->response->pass(1);
 sub patch            { Dancer::App->current->registry->universal_add('patch', @_) }
 sub path            { Dancer::FileUtils::path(@_) }
 sub post            { Dancer::App->current->registry->universal_add('post', @_) }
-sub prefix          { Dancer::App->current->set_prefix(@_) }
+sub prefix          { @_ == 0 ? Dancer::App->current->get_prefix :
+                          Dancer::App->current->set_prefix(@_) }
 sub put             { Dancer::App->current->registry->universal_add('put',     @_) }
 sub redirect        { goto &_redirect }
 sub render_with_layout { Dancer::Template::Abstract->_render_with_layout(@_) }
@@ -201,9 +206,7 @@ sub session         { goto &_session }
 sub splat           { @{ Dancer::SharedData->request->params->{splat} || [] } }
 sub start           { goto &_start }
 sub status          { Dancer::SharedData->response->status(@_) }
-sub template        { Dancer::Continuation::Route::Templated->new(
-                          return_value => Dancer::Template::Abstract->template(@_)
-                      )->throw }
+sub template        { Dancer::Template::Abstract->template(@_) }
 sub to_dumper       { Dancer::Serializer::Dumper::to_dumper(@_) }
 sub to_json         { Dancer::Serializer::JSON::to_json(@_) }
 sub to_xml          { Dancer::Serializer::XML::to_xml(@_) }
@@ -405,7 +408,7 @@ sub _send_file {
             my ( $status, $headers ) = @_;
             my %callbacks = defined $options{'callbacks'} ?
                             %{ $options{'callbacks'} }    :
-                            {};
+                            ();
 
             return sub {
                 my $respond = shift;
@@ -533,7 +536,7 @@ your app.  You can control the exporting through the normal
 L<Exporter> mechanism.  For example:
 
     # Just export the route controllers
-    use Dancer qw(before after get post put patch);
+    use Dancer qw(get post put patch del);
 
     # Export everything but pass to avoid clashing with Test::More
     use Test::More;
@@ -584,26 +587,23 @@ By default, the L<warnings> pragma will also be exported, meaning your
 app/script will be running under C<use warnings>.  If you do not want this, set
 the L<import_warnings|Dancer::Config/import_warnings> setting to a false value.
 
+=head2 !keyword
+
+If you want to simply prevent Dancer from exporting specific keywords (perhaps
+you plan to implement them yourself in a different way, or you don't plan to use
+them and they clash with another module you're loading), you can simply exclude
+them:
+
+    use Dancer qw(!session);
+
+The above would import all keywords as normal, with the exception of C<session>.
+
 
 =head1 FUNCTIONS
 
 =head2 after
 
-Add a hook at the B<after> position:
-
-    after sub {
-        my $response = shift;
-        # do something with response, e.g.:
-        $response->header('X-Beer' => 'Yes please');
-    };
-
-The anonymous function which is given to C<after> will be executed after
-having executed a route, but before the response is returned (so you can modify
-the response here, if you need to)..
-
-You can define multiple after filters, using the C<after> helper as
-many times as you wish; each filter will be executed, in the order you added
-them.
+Deprecated - see the C<after> L<hook|Dancer/hook>.
 
 =head2 any
 
@@ -621,48 +621,11 @@ Or even, a route handler that would match any HTTP methods:
 
 =head2 before
 
-Defines a before filter:
-
-    before sub {
-        # do something with request, vars or params
-    };
-
-The anonymous function given to C<before> will be executed before executing a
-route handler to handle the request.
-
-If the function modifies the request's C<path_info> or C<method>, a new
-search for a matching route is performed and the filter is re-executed.
-Considering that this can lead to an infinite loop, this mechanism
-is stopped after 10 times with an exception.
-
-The before filter can set a response with a redirection code (either
-301 or 302): in this case the matched route (if any) will be ignored and the
-redirection will be performed immediately.
-
-You can define multiple before filters, using the C<before> helper as
-many times as you wish; each filter will be executed in the order you added
-them.
+Deprecated - see the C<before> L<hook|Dancer/hook>.
 
 =head2 before_template
 
-Defines a before_template filter:
-
-    before_template sub {
-        my $tokens = shift;
-        # do something with request, vars or params
-        
-        # for example, adding a token to the template
-        $tokens->{token_name} = "some value";
-    };
-
-The anonymous function which is given to C<before_template> will be executed
-before sending data and tokens to the template. Receives a HashRef of the
-tokens that will be inserted into the template.
-
-This filter works as the C<before> and C<after> filter.
-
-Now the preferred way for this is to use C<hook>s (namely, the
-C<before_template> one). Check C<hook> documentation below.
+Deprecated - see the C<before_template> L<hook|Dancer/hook>.
 
 =head2 cookies
 
@@ -733,6 +696,12 @@ have to change the C<content_type> setting instead.
 
 Alias for the C<start> keyword.
 
+=head2 dancer_version
+
+Returns the version of Dancer. If you need the major version, do something like:
+
+  int(dancer_version);
+
 =head2 debug
 
 Logs a message of debug level:
@@ -775,13 +744,12 @@ application.
 
 It effectively lets you chain routes together in a clean manner.
 
-    get qr{ /demo/articles/(.+) }x => sub {
-        my ($article_id) = splat;
+    get '/demo/articles/:article_id' => sub {
 
         # you'll have to implement this next sub yourself :)
         change_the_main_database_to_demo();
 
-        forward '/articles/$article_id';
+        forward "/articles/" . params->{article_id};
     };
 
 In the above example, the users that reach I</demo/articles/30> will actually
@@ -792,19 +760,19 @@ database by merely going to I</demo/...>.
 
 You'll notice that in the example we didn't indicate whether it was B<GET> or
 B<POST>. That is because C<forward> chains the same type of route the user
-reached. If it was a B<GET>, it will remain a B<GET>.
+reached. If it was a B<GET>, it will remain a B<GET> (but if you do need to
+change the method, you can do so; read on below for details.)
 
-Broader functionality might be added in the future.
 
 B<WARNING> : Issuing a forward immediately exits the current route,
 and perform the forward. Thus, any code after a forward is ignored, until the
 end of the route. e.g.
 
-    get '/some/path => sub {
+    get '/foo/:article_id' => sub {
         if ($condition) {
-            forward '/articles/$article_id';
+            forward "/articles/" . params->{article_id};
             # The following code is never executed
-            do_stuf();
+            do_stuff();
         }
 
         more_stuff();
@@ -967,17 +935,17 @@ This hook receives as argument a L<Dancer::Error> object.
 
 =item before
 
-This is an alias to C<before>.
+This hook receives one argument, a reference to the subroutine that
+implements the route that is being intercepted by this hook.
 
-This hook receives no arguments.
-
-  before sub {
+  hook before => sub {
+    my $route_handler = shift;
     ...
   };
 
-is equivalent to
+it is equivalent to the deprecated
 
-  hook before => sub {
+  before sub {
     ...
   };
 
@@ -1112,6 +1080,14 @@ can be a Dancer::Exception, or a string, or whatever was used to C<die>.
 
 =back
 
+=head2 info
+
+Logs a message of info level:
+
+    info "This is a info message";
+
+See L<Dancer::Logger> for details on how to configure where log messages go.
+
 =head2 layout
 
 This method is deprecated. Use C<set>:
@@ -1120,7 +1096,7 @@ This method is deprecated. Use C<set>:
 
 =head2 logger
 
-Deprecated. Use C<<set logger => 'console'>> to change current logger engine.
+Deprecated. Use C<<set logger =E<gt> 'console'>> to change current logger engine.
 
 =head2 load
 
@@ -1687,26 +1663,46 @@ L<Dancer::HTTP/"HTTP CODES">.
 
 =head2 template
 
-Tells the route handler to build a response with the current template engine:
+Returns the response of processing the given template with the given parameters
+(and optional settings), wrapping it in the default or specified layout too, if
+layouts are in use.
+
+An example of a  route handler which returns the result of using template to 
+build a response with the current template engine:
 
     get '/' => sub {
         ...
-        template 'some_view', { token => 'value'};
+        return template 'some_view', { token => 'value'};
     };
 
-B<WARNING> : Issuing a template immediately exits the current route, and perform
-the template. Thus, any code after a template is ignored, until the end of the route.
-So it's not necessary anymore to use C<return> with template.
+Note that C<template> simply returns the content, so when you use it in a route
+handler, if execution of the the route handler should stop at that point, make
+sure you use 'return' to ensure your route handler returns the content.
 
-    get '/some/route' => sub {
+Since template just returns the result of rendering the template, you can also
+use it to perform other templating tasks, e.g. generating emails:
+
+    post '/some/route' => sub {
         if (...) {
-            # we want to let the next matching route handler process this one
-            template(...);
-            # This code will be ignored
-            do_stuff();
+            email {
+                to      => 'someone@example.com',
+                from    => 'foo@example.com',
+                subject => 'Hello there',
+                msg     => template('emails/foo', { name => params->{name} }),
+            };
+
+            return template 'message_sent';
+        } else {
+            return template 'error';
         }
     };
 
+Compatibility notice: C<template> was changed in version 1.3090 to immediately
+interrupt execution of a route handler and return the content, as it's typically
+used at the end of a route handler to return content.  However, this caused
+issues for some people who were using C<template> to generate emails etc, rather
+than accessing the template engine directly, so this change has been reverted
+in 1.3091.
 
 The first parameter should be a template available in the views directory, the
 second one (optional) is a HashRef of tokens to interpolate, and the third
@@ -1715,13 +1711,13 @@ second one (optional) is a HashRef of tokens to interpolate, and the third
 For example, to disable the layout for a specific request:
 
     get '/' => sub {
-        template 'index.tt', {}, { layout => undef };
+        template 'index', {}, { layout => undef };
     };
 
 Or to request a specific layout, of course:
 
     get '/user' => sub {
-        template 'user.tt', {}, { layout => 'user' };
+        template 'user', {}, { layout => 'user' };
     };
 
 Some tokens are automatically added to your template (C<perl_version>,
