@@ -7,6 +7,7 @@ use Carp;
 use Dancer::Logger;
 use Dancer::Factory::Hook;
 use Dancer::FileUtils 'path';
+use Dancer::Exception qw(:all);
 
 use base 'Dancer::Engine';
 
@@ -21,26 +22,40 @@ sub render { confess "render not implemented" }
 
 sub default_tmpl_ext { "tt" }
 
+# Work out the template names to look for; this will be the view name passed
+# as-is, and also the view name with the default template extension appended, if
+# it did not already end in that.
 sub _template_name {
     my ( $self, $view ) = @_;
+    my @views = ( $view );
     my $def_tmpl_ext = $self->config->{extension} || $self->default_tmpl_ext();
-    $view .= ".$def_tmpl_ext" if $view !~ /\.\Q$def_tmpl_ext\E$/;
-    return $view;
+    push @views, $view .= ".$def_tmpl_ext" if $view !~ /\.\Q$def_tmpl_ext\E$/;
+    return @views;
 }
 
 sub view {
     my ($self, $view) = @_;
 
-    $view = $self->_template_name($view);
+    my $views_dir = Dancer::App->current->setting('views');
 
-    return path(Dancer::App->current->setting('views'), $view);
+    for my $template ($self->_template_name($view)) {
+        my $view_path = path($views_dir, $template);
+        return $view_path if -f $view_path;
+    }
+
+    # No matching view path was found
+    return;
 }
 
 sub layout {
     my ($self, $layout, $tokens, $content) = @_;
 
-    my $layout_name = $self->_template_name($layout);
-    my $layout_path = path(Dancer::App->current->setting('views'), 'layouts', $layout_name);
+    my $layouts_dir = path(Dancer::App->current->setting('views'), 'layouts');
+    my $layout_path;
+    for my $layout_name ($self->_template_name($layout)) {
+        $layout_path = path($layouts_dir, $layout_name);
+        last if -e $layout_path;
+    }
 
     my $full_content;
     if (-e $layout_path) {
@@ -48,7 +63,7 @@ sub layout {
                                      $layout_path, {%$tokens, content => $content});
     } else {
         $full_content = $content;
-        Dancer::Logger::error("Defined layout ($layout_name) was not found!");
+        Dancer::Logger::error("Defined layout ($layout) was not found!");
     }
     $full_content;
 }
@@ -62,7 +77,16 @@ sub apply_renderer {
 
     Dancer::Factory::Hook->execute_hooks('before_template_render', $tokens);
 
-    my $content = $self->render($view, $tokens);
+    my $content;
+    try {
+        $content = $self->render($view, $tokens);
+    } continuation {
+        my ($continuation) = @_;
+        # If we have a Route continuation, run the after hook, then
+        # propagate the continuation
+        Dancer::Factory::Hook->execute_hooks('after_template_render', \$content);
+        $continuation->rethrow();
+    };
 
     Dancer::Factory::Hook->execute_hooks('after_template_render', \$content);
 
@@ -92,8 +116,17 @@ sub apply_layout {
 
     Dancer::Factory::Hook->execute_hooks('before_layout_render', $tokens, \$content);
 
-    my $full_content =
-      $self->layout($layout, $tokens, $content);
+    my $full_content;
+
+    try {
+        $full_content = $self->layout($layout, $tokens, $content);
+    } continuation {
+        my ($continuation) = @_;
+        # If we have a Route continuation, run the after hook, then
+        # propagate the continuation
+        Dancer::Factory::Hook->execute_hooks('after_layout_render', \$full_content);
+        $continuation->rethrow();
+    };
 
     Dancer::Factory::Hook->execute_hooks('after_layout_render', \$full_content);
 
@@ -141,11 +174,13 @@ sub template {
 
     if ($view) {
         # check if the requested view exists
-        my $view_path = $engine->view($view);
+        my $view_path = $engine->view($view) || '';
         if ($engine->view_exists($view_path)) {
             $content = $engine->apply_renderer($view, $tokens);
         } else {
-            Dancer::Logger::error("Supplied view ($view) was not found.");
+            Dancer::Logger::error(
+                "Supplied view ($view) not found - $view_path does not exist"
+            );
             return Dancer::Error->new(
                           code => 500,
                           message => 'view not found',
@@ -167,7 +202,7 @@ sub template {
     )->render();
 }
 
-sub view_exists { return -e $_[1] }
+sub view_exists { return defined $_[1] &&  -f $_[1] }
 
 1;
 __END__
@@ -255,7 +290,25 @@ configuration. For example, for the default (C<Simple>) engine:
 
 =item B<view($view)>
 
-The default behavior of this method is to return the path of the given view.
+The default behavior of this method is to return the path of the given view,
+appending the default template extension (either the value of the C<extension>
+setting in the configuration, or the value returned by C<default_tmpl_ext>) if
+it is not present in the view name given and no layout template with that exact
+name existed.  (In other words, given a layout name C<main>, if C<main> exists
+in the layouts dir, it will be used; if not, C<main.tmpl> (where C<tmpl> is the
+value of the C<extension> setting, or the value returned by C<default_tmpl_ext>)
+will be looked for.)
+
+=item B<view_exists($view_path)>
+
+By default, Dancer::Template::Abstract checks to see if it can find the
+view file calling C<view_exists($path_to_file)>. If not, it will
+generate a nice error message for the user.
+
+If you are using extending Dancer::Template::Abstract to use a template
+system with multiple document roots (like L<Text::XSlate> or
+L<Template>), you can override this method to always return true, and
+therefore skip this check.
 
 =item B<view_exists($view_path)>
 
@@ -270,7 +323,10 @@ therefore skip this check.
 
 =item B<layout($layout, $tokens, $content)>
 
-The default behavior of this method is to merge a content with a layout.
+The default behavior of this method is to merge a content with a layout.  The
+layout file is looked for with similar logic as per C<view> - an exact match
+first, then attempting to append the default template extension, if the view
+name given did not already end with it.
 
 =item B<render($self, $template, $tokens)>
 
